/* Convert ranked trees from minbp to trees w/ fixed groups
 * gp_splits B ntrees gpfile < infile
 *
 * gpfile:
 * ngps
 * gpname1 nmembers mem_name1 ...
 * gpname2 ...
 *
 * assumes minbp has been run and minbp.minbp minbp.newick have trees
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "ealgc.h"
#include "fn.h"
#include "tree2splitf.h"
#include "gp_splits_fn.h"

/*= 1 if ctree is new tree; check usplits indeces */
int check_newgptree(int numsp, int ntrees, int *tree, int *ctree){
  int tr;

  for(tr = 0; tr < ntrees; tr++)
    if (ivsame(numsp-3,&tree[tr*(numsp-3)],ctree)) return(0);
  return(1);
}

/*= convert splits to indeces of usplits; 1 if new split comes up */
int convs2uidx(int numsp, int *nspl, int *splits, int *bp_split, 
	       int *usplits, int * count, int *tree){
  int new_tree=0,tspl=0,br,spl,i;

  for(br = 0; br < numsp-3; br++){
    for(spl = 0; spl < *nspl; spl++)
      if (ivsame(numsp,&splits[br*numsp],&usplits[spl*numsp])){
	tree[tspl++] = spl;
	break;
      }
    if (spl >= *nspl){ /* split not present */
      for(i = 0; i < numsp; i++) usplits[i+(*nspl)*numsp] = splits[i+br*numsp];
      count[(*nspl)] = bp_split[br];
      tree[tspl++] = *nspl;
      (*nspl)++;
      new_tree = 1;
    }
  }
  isort(tspl,tree);
  return(new_tree);
}


/*= bp for splits given by branch lengths; compare with utreec2splits */
void gbp_splits(int numsp, int B, double *utreec, int *bp_splits){
  int join,j,r;
  for(join = 0; join < numsp-2; join++){
    for(j = 0; j < 2; j++){
      r = (int) utreec[j+join*4]; 
      if (r >= numsp)
	bp_splits[(r-numsp)] = (int) B*utreec[j+2+join*4];
    }
  }
  j = (utreec[join*4] < utreec[1+join*4])?0:1;/* special case at root */
  r = min(utreec[join*4],utreec[1+join*4]);
  if (r >= numsp)
    bp_splits[(r-numsp)] = (int) B*max(utreec[2+join*4],utreec[3+join*4]);
}

/*= main program */
int main(int argc, char **argv)
{
#define NSITE 1000
#define NUMSP 80
  FILE *gpfile,*treefile,*minbp_file;
  int ntrees,numsp,nsite,ngps,ngptree,tr,is_gpcompat,maxsplits,maxtrees,nspl;
  int is_newgptree,B,ncompat;
  int gpidx[NUMSP],gpsplits[NUMSP*NUMSP],splits[NUMSP*NUMSP],gsplits[NUMSP*NUMSP];
  int bp_splits[NUMSP],*minbp,*usplits,*tree,*count;
  char name[NUMSP][11],gpname[NUMSP][11],seqc[NSITE*NUMSP];
  double utreec[NUMSP*4];

  gpfile = fopen(argv[--argc], "r");
  ntrees = atoi(argv[--argc]);
  B = atoi(argv[--argc]);
  rinterleavef(stdin,&numsp,&nsite,name,seqc);
  ngps = gpspl_mk(gpfile,numsp,name,gpname,gpidx,gpsplits);
  fclose(gpfile);

  /* memory allocation */
  maxsplits = 100000;
  maxtrees = 1000000;
  minbp = civec(ntrees);
  usplits = civec(maxsplits*numsp);
  tree = civec(maxtrees*numsp);
  count = civec(maxsplits);

  /* get unique splits */
  treefile = fopen("minbp.newick", "r");
  minbp_file = fopen("minbp.minbp", "r");
  ngptree = 0; nspl = 0; ncompat = 0;
  for(tr = 0; tr < ntrees; tr++){ 
    fscanf(minbp_file, "%i", &minbp[tr]);
    read_tree2nutreec(treefile,numsp,name,utreec);
    utreec2splits(numsp,utreec,splits);
    gbp_splits(numsp,B,utreec,bp_splits);
    is_gpcompat = conv2gpsplits(numsp,ngps,splits,gpidx,gpsplits,bp_splits,gsplits);
    if (is_gpcompat){
      ncompat++;
      is_newgptree = convs2uidx(ngps,&nspl,gsplits,bp_splits,usplits,count,
				&tree[ngptree*(ngps-3)]);
      if (!is_newgptree)
	is_newgptree = check_newgptree(ngps,ngptree,tree,&tree[ngptree*(ngps-3)]);
      if (is_newgptree)
	minbp[ngptree++] = minbp[tr];
    }
  }
  
  printf("There were %i trees that had splits incompatible with the groups\n", ntrees-ncompat);
  printf("There were %i trees of groups\n", ngptree);
  fclose(treefile); fclose(minbp_file);
  
  treefile = fopen("minbp.newick_new", "w");
  minbp_file = fopen("minbp.minbp_new", "w");
  for(tr = 0; tr < ngptree; tr++){
    splits2tree(B,ngps,nspl,&tree[tr*(ngps-3)],usplits,count,utreec);
    pr_newick(treefile,ngps,gpname,utreec);
    fprintf(minbp_file,"%i\n",minbp[tr]);
  }
  fclose(treefile); fclose(minbp_file);

  free(count); free(minbp); free(usplits); free(tree);
  
  return(0);
}
