/* Obtain all of the trees greater than or equal to a threshold. Optionally, 
 * that threshold can be determined through double bootstrapping.
 *
 * minBP zeta alpha B treefile < infile
 *
 * zeta: cutoff or -1 if determined from double boostrap
 * If zeta is a cutoff, it should be an integer representing the number of times
 * a split occurs in bootstrap samples. For instance, zeta=50 and B=1000 would
 * mean that any tree for which the minimally supported split arose at least 50
 * times in the 1000 bootstrap samples would be output.
 *
 * treefile: In the case that zeta = -1, should contain NEWICK format trees
 * estimated for the following data sets:
 * 
 *   original data
 *   b1: bootstrap data set 1 of original data
 *           ....
 *   bB: bootstrap data set B of original data
 *   bootstrap data set 1 of data b1
 *           ....
 *   bootstrap data set B of data b1
 *   bootstrap data set 1 of data b2
 *           ....
 *   bootstrap data set B of data bB
 *
 * If zeta >= 0, treefile should contain NEWICK format trees estimated for the
 * original data and B bootstrap data sets from the original data.
 *
 * outfiles:
 *
 * minbp.newick: The trees with minBP exceeding the threshold, ordered from the
 *  tree with largest minBP to smallest
 *
 * minbp.minbp: The minBP values for the trees in minbp.newick
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "fn.h"
#include "tree2splitf.h"
#include "ealgc.h"
int minbp_distz(FILE *treefile, int B, int numsp, char name[][11], int *splits,
		int *usplits, int *count, int *minbp);

int main(int argc, char **argv)
{
#define NUMSP 80
  FILE *newick_outfile,*treefile,*outfile;
  int tr,i,numsp,nsite,B,nspl,nsplc,zeta,maxtrees,ntrees,*count,*minbp,*trees;
  int *minbpv,*o;
  int *usplits,*splits;
  char name[NUMSP][11],*seqc;
  double alpha,*utreec;
  
  treefile = fopen(argv[--argc],"r");
  B = atoi(argv[--argc]);
  alpha = atof(argv[--argc]);
  zeta = atoi(argv[--argc]);

  maxtrees = 1000000;
  scanf("%i %i", &numsp,&nsite);
  seqc = (char *) malloc((size_t) nsite*numsp*sizeof(char));
  rinterleavef(stdin,&numsp,&nsite,name,seqc);
  minbp=(int *) malloc((size_t) B*sizeof(int));
  splits=(int *) malloc((size_t) numsp*(numsp-3)*5*sizeof(int));
  nspl=dim_split(numsp,B);
  usplits=(int *) malloc((size_t) nspl*numsp*sizeof(int));
  count=(int *) malloc((size_t) nspl*sizeof(int));
  trees = (int *) malloc((size_t) maxtrees*(numsp-3)*sizeof(int));
  utreec =(double *) malloc((size_t) (numsp-1)*4*sizeof(double));
  minbpv = (int *) malloc((size_t) maxtrees*sizeof(int));
  o = (int *) malloc((size_t) maxtrees*sizeof(int));

  if (zeta < 0){
    nspl = minbp_dist(treefile,B,numsp,name,splits,usplits,count,minbp);
    zeta = minbp_cut(B,alpha,minbp);    
    printf("With B=%i, the minimum bootstrap support cutoff for a %.1f%% confidence region is %i\n", B, (1-alpha)*100, zeta);
  }
  else{
    nspl = usplitf(treefile,B,numsp,name,splits,usplits,count);
  }
  fclose(treefile);

  nsplc = sort_splits(numsp,nspl,zeta,usplits,count);
  ntrees = compat_splits_mem(B,numsp,nsplc,maxtrees,zeta,usplits,count,trees);
  if (ntrees >= maxtrees){
    printf("Maximum number of trees exceeded\n");
    exit(1);
  }
  minbp_trees(numsp,ntrees,trees,count,minbpv);
  for(i = 0; i < ntrees; i++) minbpv[i] = -minbpv[i];
  iorder(ntrees,minbpv,o);
  for(i = 0; i < ntrees; i++) minbpv[i] = -minbpv[i];

  printf("%i trees have minBP greater than or equal to %i\n", ntrees, zeta);
  newick_outfile = fopen("minbp.newick","w");
  for(tr = 0; tr < ntrees; tr++){
    splits2tree(B,numsp,nspl,&trees[o[tr]*(numsp-3)],usplits,count,utreec);
    pr_newick(newick_outfile, numsp, name, utreec);
  }
  fclose(newick_outfile);

  outfile = fopen("minbp.minbp","w");
  for(tr = 0; tr < ntrees; tr++) fprintf(outfile,"%i\n",minbpv[o[tr]]);
  fclose(outfile);
  
  free(usplits); free(count); free(splits); free(minbp); free(trees); free(utreec);
  free(o); free(minbpv);
  return(0);
}
